@@ -2,7 +2,7 @@
 JSONRPC Dispatcher
 ------------------
 
-This module contains the jsonrpc dispatcher class.
+This module implements a JSON 1.0 compatible dispatcher
 
 see http://json-rpc.org/wiki/specification
 '''
@@ -21,6 +21,11 @@ except ImportError:
     except ImportError:
         import json
 
+# indent the json output by this many characters
+# 0 does newlines only and None does most compact
+# This is consistent with SimpleXMLRPCServer output
+JSON_INDENT = 0
+
 # These error codes may be defined by the json-rpc spec at a later date
 # see http://json-rpc.org/wd/JSON-RPC-1-1-WD-20060807.html#ErrorObject
 JSONRPC_SERVER_ERROR = 100
@@ -63,7 +68,7 @@ class JSONRPCDispatcher:
             res['result'] = None
             
         try:
-            return json.dumps(res)
+            return json.dumps(res, indent=JSON_INDENT)
         except:
             err = {'message': 'failed to encode return value',
                    'code': JSONRPC_SERVICE_ERROR,
@@ -71,7 +76,7 @@ class JSONRPCDispatcher:
                 
             res['result'] = None
             res['error'] = err
-            return json.dumps(res)
+            return json.dumps(res, indent=JSON_INDENT)
     
     def dispatch(self, json_data):
         '''
@@ -137,7 +142,8 @@ class JSONRPCDispatcher:
             return self._encode_result(jsondict.get('id', ''), result, None)
         else:
             return self._encode_result(jsondict.get('id', ''), None, 
-                    {'message': 'method not found',  
+                    {'message': 'method "' + jsondict['method'] + \
+                     '" is not supported',  
                      'code': JSONRPC_PROCEDURE_NOT_FOUND_ERROR})
 
     
\ No newline at end of file
