@@ -11,8 +11,9 @@ It also contains a decorator to mark methods as rpc methods.
 import platform
 import inspect
 import pydoc
+import xmlrpclib
 from SimpleXMLRPCServer import SimpleXMLRPCDispatcher
-from jsonrpcdispatcher import JSONRPCDispatcher
+from jsonrpcdispatcher import JSONRPCDispatcher, json
 from xmlrpclib import Fault
 
 # this error code is taken from xmlrpc-epi
@@ -26,11 +27,13 @@ def rpcmethod(**kwargs):
     EXAMPLES:
     @rpcmethod()
     @rpcmethod(name='myns.myFuncName', signature=['int','int'])
+    @rpcmethod(permission='add_group')
     '''
     
     def set_rpcmethod_info(method):
         method.is_rpcmethod = True
         method.signature = []
+        method.permission = None
         method.external_name = getattr(method, '__name__')
 
         if 'name' in kwargs:
@@ -38,6 +41,9 @@ def rpcmethod(**kwargs):
 
         if 'signature' in kwargs:
             method.signature = kwargs['signature']
+            
+        if 'permission' in kwargs:
+            method.permission = kwargs['permission']
 
         return method
     return set_rpcmethod_info
@@ -52,6 +58,7 @@ class RPCMethod:
         self.help = ''
         self.signature = []
         self.name = ''
+        self.permission = None
         self.args = []
         
         # set the method name based on @rpcmethod or the passed value
@@ -69,6 +76,9 @@ class RPCMethod:
         else:
             self.help = pydoc.getdoc(method)
             
+        # set the permissions based on the decorator
+        self.permission = getattr(method, 'permission', None)
+            
         # use inspection (reflection) to get the arguments
         args, varargs, keywords, defaults = inspect.getargspec(method)
         self.args = [arg for arg in args if arg != 'self']
@@ -144,7 +154,7 @@ class RPCDispatcher:
     def __init__(self, url='', apps=[], restrict_introspection=False):
         version = platform.python_version_tuple()
         self.url = url
-        self.rpcmethods = []
+        self.rpcmethods = []        # a list of RPCMethod objects
         self.jsonrpcdispatcher = JSONRPCDispatcher()
         
         if int(version[0]) < 3 and int(version[1]) < 5:
@@ -249,6 +259,31 @@ class RPCDispatcher:
         
         return self.xmlrpcdispatcher._marshaled_dispatch(raw_post_data)
         
+    def get_method_name(self, raw_post_data, request_format='xml'):
+        '''
+        Gets the name of the method to be called given the post data
+        and the format of the data
+        '''
+        
+        if request_format == 'xml':
+            # xmlrpclib.loads could throw an exception, but this is fine
+            # since _marshaled_dispatch would throw the same thing
+            try:
+                params, method = xmlrpclib.loads(raw_post_data)
+                return method
+            except Fault:
+                return None
+        else:
+            try:
+                # attempt to do a json decode on the data
+                jsondict = json.loads(raw_post_data)
+                if not isinstance(jsondict, dict) or 'method' not in jsondict:
+                    return None
+                else:
+                    return jsondict['method']
+            except ValueError:
+                return None
+        
     def list_methods(self):
         '''
         Returns a list of RPCMethod objects supported by the server
